!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2010.  Los Alamos National Security, LLC. This material was    !
! produced under U.S. Government contract DE-AC52-06NA25396 for Los Alamos !
! National Laboratory (LANL), which is operated by Los Alamos National     !
! Security, LLC for the U.S. Department of Energy. The U.S. Government has !
! rights to use, reproduce, and distribute this software.  NEITHER THE     !
! GOVERNMENT NOR LOS ALAMOS NATIONAL SECURITY, LLC MAKES ANY WARRANTY,     !
! EXPRESS OR IMPLIED, OR ASSUMES ANY LIABILITY FOR THE USE OF THIS         !
! SOFTWARE.  If software is modified to produce derivative works, such     !
! modified software should be clearly marked, so as not to confuse it      !
! with the version available from LANL.                                    !
!                                                                          !
! Additionally, this program is free software; you can redistribute it     !
! and/or modify it under the terms of the GNU General Public License as    !
! published by the Free Software Foundation; version 2.0 of the License.   !
! Accordingly, this program is distributed in the hope that it will be     !
! useful, but WITHOUT ANY WARRANTY; without even the implied warranty of   !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General !
! Public License for more details.                                         !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE WRTRESTARTLIB(ITER)

  USE kspacearray
  USE neblistarray
  USE ppotarray
  USE restartarray
  USE sparsearray
  USE univarray
  USE xboarray
  USE coulombarray
  USE diagarray
  USE mdarray
  USE nonoarray
  USE purearray
  USE setuparray
  USE spinarray
  USE virialarray

  USE CONSTANTS_MOD
#ifdef MPI_ON
  USE MPI
#endif

  IMPLICIT NONE

  INTEGER :: I, K, ITER
  INTEGER :: MYID, IERR
  INTEGER :: CHARACTERCHECK(9),COMPLEXCHECK(15),INTEGERCHECK(20),REALCHECK(99)
  COMPLEX(LATTEPREC) :: KSUM
  REAL(LATTEPREC), ALLOCATABLE :: TMPBODIAGUP(:), TMPBODIAGDOWN(:)
  CHARACTER(LEN=100) :: FLNM, NAME

  IF (EXISTERROR) RETURN

  IF (ITER .LT. 10) THEN
     WRITE(FLNM,'("restart.latte.", I1,".dat")') ITER
  ELSEIF (ITER .GE. 10 .AND. ITER .LT. 100) THEN
     WRITE(FLNM,'("restart.latte.", I2,".dat")') ITER
  ELSEIF (ITER .GE. 100 .AND. ITER .LT. 1000) THEN
     WRITE(FLNM,'("restart.latte.", I3,".dat")') ITER
  ELSEIF (ITER .GE. 1000 .AND. ITER .LT. 10000) THEN
     WRITE(FLNM,'("restart.latte.", I4,".dat")') ITER
  ELSEIF (ITER .GE. 10000 .AND. ITER .LT. 100000) THEN
     WRITE(FLNM,'("restart.latte.", I5,".dat")') ITER
  ELSEIF (ITER .GE. 100000 .AND. ITER .LT. 1000000) THEN
     WRITE(FLNM,'("restart.latte.", I6,".dat")') ITER
  ELSEIF (ITER .GE. 1000000 .AND. ITER .LT. 10000000) THEN
     WRITE(FLNM,'("restart.latte.", I7,".dat")') ITER
  ENDIF

  OPEN(14,FILE=FLNM,FORM="UNFORMATTED",ACCESS="SEQUENTIAL")

  CHARACTERCHECK = 0
  INTEGERCHECK = 0
  COMPLEXCHECK = 0
  REALCHECK = 0

  IF(ALLOCATED( ATELE ) .AND. RSLEVEL == 0) CHARACTERCHECK( 2 ) = 1

  IF(ALLOCATED( BASIS ) .AND. RSLEVEL == 0) CHARACTERCHECK( 3 ) = 1

  IF(ALLOCATED( BTYPE ) .AND. RSLEVEL == 0) CHARACTERCHECK( 4 ) = 1

  IF(ALLOCATED( ELE ) .AND. RSLEVEL == 0) CHARACTERCHECK( 5 ) = 1

  IF(ALLOCATED( ELE1 ) .AND. RSLEVEL == 0) CHARACTERCHECK( 6 ) = 1

  IF(ALLOCATED( ELE2 ) .AND. RSLEVEL == 0) CHARACTERCHECK( 7 ) = 1

  IF(ALLOCATED( PPELE1 ) .AND. RSLEVEL == 0) CHARACTERCHECK( 8 ) = 1

  IF(ALLOCATED( PPELE2 ) .AND. RSLEVEL == 0) CHARACTERCHECK( 9 ) = 1

  IF(ALLOCATED( DIAG_ZWORK ) .AND. RSLEVEL == 0) COMPLEXCHECK( 2 ) = 1

  IF(ALLOCATED( HK ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 3 ) = 1

  IF(ALLOCATED( HK0 ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 4 ) = 1

  IF(ALLOCATED( HKDIAG ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 5 ) = 1

  IF(ALLOCATED( KBO ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 6 ) = 1

  IF(ALLOCATED( KEVECS ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 7 ) = 1

  IF(ALLOCATED( KF ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 8 ) = 1

  IF(ALLOCATED( KHTMP ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 9 ) = 1

  IF(ALLOCATED( KORTHOH ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 10 ) = 1

  IF(ALLOCATED( KXMAT ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 11 ) = 1

  IF(ALLOCATED( SK ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 12 ) = 1

  IF(ALLOCATED( ZBO ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 13 ) = 1

  IF(ALLOCATED( ZHEEVD_WORK ) .AND. RSLEVEL == 0) COMPLEXCHECK( 14 ) = 1

  IF(ALLOCATED( ZHJJ ) .AND. RSLEVEL == 0) COMPLEXCHECK( 15 ) = 1

  IF(ALLOCATED( DIAG_IWORK ) .AND. RSLEVEL == 0) INTEGERCHECK( 2 ) = 1

  IF(ALLOCATED( ELEMPOINTER ) .AND. RSLEVEL == 0) INTEGERCHECK( 3 ) = 1

  IF(ALLOCATED( IFAIL ) .AND. RSLEVEL == 0) INTEGERCHECK( 4 ) = 1

  IF(ALLOCATED( MATINDLIST ) .AND. RSLEVEL == 0) INTEGERCHECK( 5 ) = 1

  IF(ALLOCATED( MOLID ) .AND. RSLEVEL == 0) INTEGERCHECK( 6 ) = 1

  IF(ALLOCATED( NEBCOUL ) .AND. RSLEVEL >= 2) INTEGERCHECK( 7 ) = 1

  IF(ALLOCATED( NEBPP ) .AND. RSLEVEL >= 2) INTEGERCHECK( 8 ) = 1

  IF(ALLOCATED( NEBTB ) .AND. RSLEVEL >= 2) INTEGERCHECK( 9 ) = 1

  IF(ALLOCATED( NONO_IWORK ) .AND. RSLEVEL == 0) INTEGERCHECK( 10 ) = 1

  IF(ALLOCATED( PPTABLENGTH ) .AND. RSLEVEL == 0) INTEGERCHECK( 11 ) = 1

  IF(ALLOCATED( RX ) .AND. RSLEVEL == 0) INTEGERCHECK( 12 ) = 1

  IF(ALLOCATED( RXTMP ) .AND. RSLEVEL == 0) INTEGERCHECK( 13 ) = 1

  IF(ALLOCATED( SIGNLIST ) .AND. RSLEVEL == 0) INTEGERCHECK( 14 ) = 1

  IF(ALLOCATED( SPININDLIST ) .AND. RSLEVEL == 0) INTEGERCHECK( 15 ) = 1

  IF(ALLOCATED( TOTNEBCOUL ) .AND. RSLEVEL == 0) INTEGERCHECK( 16 ) = 1

  IF(ALLOCATED( TOTNEBPP ) .AND. RSLEVEL == 0) INTEGERCHECK( 17 ) = 1

  IF(ALLOCATED( TOTNEBTB ) .AND. RSLEVEL == 0) INTEGERCHECK( 18 ) = 1

  IF(ALLOCATED( XB ) .AND. RSLEVEL == 0) INTEGERCHECK( 19 ) = 1

  IF(ALLOCATED( ZHEEVD_IWORK ) .AND. RSLEVEL == 0) INTEGERCHECK( 20 ) = 1

  IF(ALLOCATED( ATOCC ) .AND. RSLEVEL == 0) REALCHECK( 2 ) = 1

  IF(ALLOCATED( BO ) .AND. RSLEVEL >= 1) REALCHECK( 3 ) = 1

  IF(ALLOCATED( BOND ) .AND. RSLEVEL >= 1) REALCHECK( 4 ) = 1

  IF(ALLOCATED( BOZERO ) .AND. RSLEVEL == 0) REALCHECK( 5 ) = 1

  IF(ALLOCATED( BO_PADDED ) .AND. RSLEVEL >= 1) REALCHECK( 6 ) = 1

  IF(ALLOCATED( CHEMPOT_PNK ) .AND. RSLEVEL == 0) REALCHECK( 7 ) = 1

  IF(ALLOCATED( COSLIST ) .AND. RSLEVEL == 0) REALCHECK( 8 ) = 1

  IF(ALLOCATED( COULOMBV ) .AND. RSLEVEL == 0) REALCHECK( 9 ) = 1

  IF(ALLOCATED( CPLIST ) .AND. RSLEVEL == 0) REALCHECK( 10 ) = 1

  IF(ALLOCATED( CR ) .AND. RSLEVEL >= 1) REALCHECK( 11 ) = 1

  IF(ALLOCATED( DELTAQ ) .AND. RSLEVEL == 0) REALCHECK( 12 ) = 1

  IF(ALLOCATED( DELTASPIN ) .AND. RSLEVEL == 0) REALCHECK( 13 ) = 1

  IF(ALLOCATED( DIAG_RWORK ) .AND. RSLEVEL == 0) REALCHECK( 14 ) = 1

  IF(ALLOCATED( DIAG_WORK ) .AND. RSLEVEL == 0) REALCHECK( 15 ) = 1

  IF(ALLOCATED( DOWNEVALS ) .AND. RSLEVEL == 0) REALCHECK( 16 ) = 1

  IF(ALLOCATED( DOWNEVECS ) .AND. RSLEVEL >= 1) REALCHECK( 17 ) = 1

  IF(ALLOCATED( EHIST ) .AND. RSLEVEL == 0) REALCHECK( 18 ) = 1

  IF(ALLOCATED( EVALS ) .AND. RSLEVEL == 0) REALCHECK( 19 ) = 1

  IF(ALLOCATED( EVECS ) .AND. RSLEVEL >= 1) REALCHECK( 20 ) = 1

  IF(ALLOCATED( F ) .AND. RSLEVEL >= 1) REALCHECK( 21 ) = 1

  IF(ALLOCATED( FCOUL ) .AND. RSLEVEL >= 1) REALCHECK( 22 ) = 1

  IF(ALLOCATED( FPP ) .AND. RSLEVEL >= 1) REALCHECK( 23 ) = 1

  IF(ALLOCATED( FPUL ) .AND. RSLEVEL >= 1) REALCHECK( 24 ) = 1

  IF(ALLOCATED( FRANPREV ) .AND. RSLEVEL >= 1) REALCHECK( 25 ) = 1

  IF(ALLOCATED( FSCOUL ) .AND. RSLEVEL >= 1) REALCHECK( 26 ) = 1

  IF(ALLOCATED( FSSPIN ) .AND. RSLEVEL >= 1) REALCHECK( 27 ) = 1

  IF(ALLOCATED( FTOT ) .AND. RSLEVEL >= 1) REALCHECK( 28 ) = 1

  IF(ALLOCATED( H ) .AND. RSLEVEL >= 1) REALCHECK( 29 ) = 1

  IF(ALLOCATED( H0 ) .AND. RSLEVEL >= 1) REALCHECK( 30 ) = 1

  IF(ALLOCATED( H2VECT ) .AND. RSLEVEL == 0) REALCHECK( 31 ) = 1

  IF(ALLOCATED( HDIAG ) .AND. RSLEVEL == 0) REALCHECK( 32 ) = 1

  IF(ALLOCATED( HDOWN ) .AND. RSLEVEL >= 1) REALCHECK( 33 ) = 1

  IF(ALLOCATED( HED ) .AND. RSLEVEL == 0) REALCHECK( 34 ) = 1

  IF(ALLOCATED( HEF ) .AND. RSLEVEL == 0) REALCHECK( 35 ) = 1

  IF(ALLOCATED( HEP ) .AND. RSLEVEL == 0) REALCHECK( 36 ) = 1

  IF(ALLOCATED( HES ) .AND. RSLEVEL == 0) REALCHECK( 37 ) = 1

  IF(ALLOCATED( HJJ ) .AND. RSLEVEL == 0) REALCHECK( 38 ) = 1

  IF(ALLOCATED( HR0 ) .AND. RSLEVEL == 0) REALCHECK( 39 ) = 1

  IF(ALLOCATED( HUBBARDU ) .AND. RSLEVEL == 0) REALCHECK( 40 ) = 1

  IF(ALLOCATED( HUP ) .AND. RSLEVEL >= 1) REALCHECK( 41 ) = 1

  IF(ALLOCATED( KEVALS ) .AND. RSLEVEL >= 1) REALCHECK( 42 ) = 1

  IF(ALLOCATED( LCNSHIFT ) .AND. RSLEVEL == 0) REALCHECK( 43 ) = 1

  IF(ALLOCATED( MASS ) .AND. RSLEVEL == 0) REALCHECK( 44 ) = 1

  IF(ALLOCATED( MYCHARGE ) .AND. RSLEVEL == 0) REALCHECK( 45 ) = 1

  IF(ALLOCATED( NONOTMP ) .AND. RSLEVEL >= 1) REALCHECK( 46 ) = 1

  IF(ALLOCATED( NONO_EVALS ) .AND. RSLEVEL == 0) REALCHECK( 47 ) = 1

  IF(ALLOCATED( NONO_WORK ) .AND. RSLEVEL == 0) REALCHECK( 48 ) = 1

  IF(ALLOCATED( OLDDELTAQS ) .AND. RSLEVEL == 0) REALCHECK( 49 ) = 1

  IF(ALLOCATED( OLDDELTASPIN ) .AND. RSLEVEL == 0) REALCHECK( 50 ) = 1

  IF(ALLOCATED( ORTHOH ) .AND. RSLEVEL >= 1) REALCHECK( 51 ) = 1

  IF(ALLOCATED( ORTHOHDOWN ) .AND. RSLEVEL >= 1) REALCHECK( 52 ) = 1

  IF(ALLOCATED( ORTHOHUP ) .AND. RSLEVEL >= 1) REALCHECK( 53 ) = 1

  IF(ALLOCATED( ORTHORHO ) .AND. RSLEVEL >= 1) REALCHECK( 54 ) = 1

  IF(ALLOCATED( OVERL ) .AND. RSLEVEL >= 1) REALCHECK( 55 ) = 1

  IF(ALLOCATED( PAIR ) .AND. RSLEVEL >= 1) REALCHECK( 56 ) = 1

  IF(ALLOCATED( PHIST ) .AND. RSLEVEL == 0) REALCHECK( 57 ) = 1

  IF(ALLOCATED( PHISTX ) .AND. RSLEVEL == 0) REALCHECK( 58 ) = 1

  IF(ALLOCATED( PHISTY ) .AND. RSLEVEL == 0) REALCHECK( 59 ) = 1

  IF(ALLOCATED( PHISTZ ) .AND. RSLEVEL == 0) REALCHECK( 60 ) = 1

  IF(ALLOCATED( PNK ) .AND. RSLEVEL >= 1) REALCHECK( 61 ) = 1

  IF(ALLOCATED( POTCOEF ) .AND. RSLEVEL >= 1) REALCHECK( 62 ) = 1

  IF(ALLOCATED( PPR ) .AND. RSLEVEL >= 1) REALCHECK( 63 ) = 1

  IF(ALLOCATED( PPSPL ) .AND. RSLEVEL >= 1) REALCHECK( 64 ) = 1

  IF(ALLOCATED( PPVAL ) .AND. RSLEVEL >= 1) REALCHECK( 65 ) = 1

  IF(ALLOCATED( QLIST ) .AND. RSLEVEL == 0) REALCHECK( 66 ) = 1

  IF(ALLOCATED( RESPCHI ) .AND. RSLEVEL == 0) REALCHECK( 67 ) = 1

  IF(ALLOCATED( RHODOWN ) .AND. RSLEVEL >= 1) REALCHECK( 68 ) = 1

  IF(ALLOCATED( RHODOWNZERO ) .AND. RSLEVEL == 0) REALCHECK( 69 ) = 1

  IF(ALLOCATED( RHOUP ) .AND. RSLEVEL >= 1) REALCHECK( 70 ) = 1

  IF(ALLOCATED( RHOUPZERO ) .AND. RSLEVEL == 0) REALCHECK( 71 ) = 1

  IF(ALLOCATED( SH2 ) .AND. RSLEVEL >= 1) REALCHECK( 72 ) = 1

  IF(ALLOCATED( SINLIST ) .AND. RSLEVEL == 0) REALCHECK( 73 ) = 1

  IF(ALLOCATED( SMAT ) .AND. RSLEVEL >= 1) REALCHECK( 74 ) = 1

  IF(ALLOCATED( SPINLIST ) .AND. RSLEVEL == 0) REALCHECK( 75 ) = 1

  IF(ALLOCATED( SPINTMP ) .AND. RSLEVEL >= 1) REALCHECK( 76 ) = 1

  IF(ALLOCATED( SPIN_PNK ) .AND. RSLEVEL >= 1) REALCHECK( 77 ) = 1

  IF(ALLOCATED( THIST ) .AND. RSLEVEL == 0) REALCHECK( 78 ) = 1

  IF(ALLOCATED( TMPBODIAG ) .AND. RSLEVEL == 0) REALCHECK( 79 ) = 1

  IF(ALLOCATED( TMPRHODOWN ) .AND. RSLEVEL == 0) REALCHECK( 80 ) = 1

  IF(ALLOCATED( TMPRHOUP ) .AND. RSLEVEL == 0) REALCHECK( 81 ) = 1

  IF(ALLOCATED( TWOXX2 ) .AND. RSLEVEL >= 1) REALCHECK( 82 ) = 1

  IF(ALLOCATED( UMAT ) .AND. RSLEVEL >= 1) REALCHECK( 83 ) = 1

  IF(ALLOCATED( UPEVALS ) .AND. RSLEVEL == 0) REALCHECK( 84 ) = 1

  IF(ALLOCATED( UPEVECS ) .AND. RSLEVEL >= 1) REALCHECK( 85 ) = 1

  IF(ALLOCATED( V ) .AND. RSLEVEL >= 1) REALCHECK( 86 ) = 1

  IF(ALLOCATED( ATELE ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 2 ) = 1

  IF(ALLOCATED( BASIS ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 3 ) = 1

  IF(ALLOCATED( BTYPE ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 4 ) = 1

  IF(ALLOCATED( ELE ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 5 ) = 1

  IF(ALLOCATED( ELE1 ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 6 ) = 1

  IF(ALLOCATED( ELE2 ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 7 ) = 1

  IF(ALLOCATED( PPELE1 ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 8 ) = 1

  IF(ALLOCATED( PPELE2 ) .AND. RSLEVEL >= 0) CHARACTERCHECK( 9 ) = 1

  IF(ALLOCATED( DIAG_ZWORK ) .AND. RSLEVEL >= 0) COMPLEXCHECK( 2 ) = 1

  IF(ALLOCATED( HK ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 3 ) = 1

  IF(ALLOCATED( HK0 ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 4 ) = 1

  IF(ALLOCATED( HKDIAG ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 5 ) = 1

  IF(ALLOCATED( KBO ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 6 ) = 1

  IF(ALLOCATED( KEVECS ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 7 ) = 1

  IF(ALLOCATED( KF ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 8 ) = 1

  IF(ALLOCATED( KHTMP ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 9 ) = 1

  IF(ALLOCATED( KORTHOH ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 10 ) = 1

  IF(ALLOCATED( KXMAT ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 11 ) = 1

  IF(ALLOCATED( SK ) .AND. RSLEVEL >= 2) COMPLEXCHECK( 12 ) = 1

  IF(ALLOCATED( ZBO ) .AND. RSLEVEL >= 1) COMPLEXCHECK( 13 ) = 1

  IF(ALLOCATED( ZHEEVD_WORK ) .AND. RSLEVEL >= 0) COMPLEXCHECK( 14 ) = 1

  IF(ALLOCATED( ZHJJ ) .AND. RSLEVEL >= 0) COMPLEXCHECK( 15 ) = 1

  IF(ALLOCATED( DIAG_IWORK ) .AND. RSLEVEL >= 0) INTEGERCHECK( 2 ) = 1

  IF(ALLOCATED( ELEMPOINTER ) .AND. RSLEVEL >= 0) INTEGERCHECK( 3 ) = 1

  IF(ALLOCATED( IFAIL ) .AND. RSLEVEL >= 0) INTEGERCHECK( 4 ) = 1

  IF(ALLOCATED( MATINDLIST ) .AND. RSLEVEL >= 0) INTEGERCHECK( 5 ) = 1

  IF(ALLOCATED( MOLID ) .AND. RSLEVEL >= 0) INTEGERCHECK( 6 ) = 1

  IF(ALLOCATED( NEBCOUL ) .AND. RSLEVEL >= 2) INTEGERCHECK( 7 ) = 1

  IF(ALLOCATED( NEBPP ) .AND. RSLEVEL >= 2) INTEGERCHECK( 8 ) = 1

  IF(ALLOCATED( NEBTB ) .AND. RSLEVEL >= 2) INTEGERCHECK( 9 ) = 1

  IF(ALLOCATED( NONO_IWORK ) .AND. RSLEVEL >= 0) INTEGERCHECK( 10 ) = 1

  IF(ALLOCATED( PPTABLENGTH ) .AND. RSLEVEL >= 0) INTEGERCHECK( 11 ) = 1

  IF(ALLOCATED( RX ) .AND. RSLEVEL >= 0) INTEGERCHECK( 12 ) = 1

  IF(ALLOCATED( RXTMP ) .AND. RSLEVEL >= 0) INTEGERCHECK( 13 ) = 1

  IF(ALLOCATED( SIGNLIST ) .AND. RSLEVEL >= 0) INTEGERCHECK( 14 ) = 1

  IF(ALLOCATED( SPININDLIST ) .AND. RSLEVEL >= 0) INTEGERCHECK( 15 ) = 1

  IF(ALLOCATED( TOTNEBCOUL ) .AND. RSLEVEL >= 0) INTEGERCHECK( 16 ) = 1

  IF(ALLOCATED( TOTNEBPP ) .AND. RSLEVEL >= 0) INTEGERCHECK( 17 ) = 1

  IF(ALLOCATED( TOTNEBTB ) .AND. RSLEVEL >= 0) INTEGERCHECK( 18 ) = 1

  IF(ALLOCATED( XB ) .AND. RSLEVEL >= 0) INTEGERCHECK( 19 ) = 1

  IF(ALLOCATED( ZHEEVD_IWORK ) .AND. RSLEVEL >= 0) INTEGERCHECK( 20 ) = 1

  IF(ALLOCATED( ATOCC ) .AND. RSLEVEL >= 0) REALCHECK( 2 ) = 1

  IF(ALLOCATED( BO ) .AND. RSLEVEL >= 1) REALCHECK( 3 ) = 1

  IF(ALLOCATED( BOND ) .AND. RSLEVEL >= 1) REALCHECK( 4 ) = 1

  IF(ALLOCATED( BOZERO ) .AND. RSLEVEL >= 0) REALCHECK( 5 ) = 1

  IF(ALLOCATED( BO_PADDED ) .AND. RSLEVEL >= 1) REALCHECK( 6 ) = 1

  IF(ALLOCATED( CHEMPOT_PNK ) .AND. RSLEVEL >= 0) REALCHECK( 7 ) = 1

  IF(ALLOCATED( COSLIST ) .AND. RSLEVEL >= 0) REALCHECK( 8 ) = 1

  IF(ALLOCATED( COULOMBV ) .AND. RSLEVEL >= 0) REALCHECK( 9 ) = 1

  IF(ALLOCATED( CPLIST ) .AND. RSLEVEL >= 0) REALCHECK( 10 ) = 1

  IF(ALLOCATED( CR ) .AND. RSLEVEL >= 1) REALCHECK( 11 ) = 1

  IF(ALLOCATED( DELTAQ ) .AND. RSLEVEL >= 0) REALCHECK( 12 ) = 1

  IF(ALLOCATED( DELTASPIN ) .AND. RSLEVEL >= 0) REALCHECK( 13 ) = 1

  IF(ALLOCATED( DIAG_RWORK ) .AND. RSLEVEL >= 0) REALCHECK( 14 ) = 1

  IF(ALLOCATED( DIAG_WORK ) .AND. RSLEVEL >= 0) REALCHECK( 15 ) = 1

  IF(ALLOCATED( DOWNEVALS ) .AND. RSLEVEL >= 0) REALCHECK( 16 ) = 1

  IF(ALLOCATED( DOWNEVECS ) .AND. RSLEVEL >= 1) REALCHECK( 17 ) = 1

  IF(ALLOCATED( EHIST ) .AND. RSLEVEL >= 0) REALCHECK( 18 ) = 1

  IF(ALLOCATED( EVALS ) .AND. RSLEVEL >= 0) REALCHECK( 19 ) = 1

  IF(ALLOCATED( EVECS ) .AND. RSLEVEL >= 1) REALCHECK( 20 ) = 1

  IF(ALLOCATED( F ) .AND. RSLEVEL >= 1) REALCHECK( 21 ) = 1

  IF(ALLOCATED( FCOUL ) .AND. RSLEVEL >= 1) REALCHECK( 22 ) = 1

  IF(ALLOCATED( FPP ) .AND. RSLEVEL >= 1) REALCHECK( 23 ) = 1

  IF(ALLOCATED( FPUL ) .AND. RSLEVEL >= 1) REALCHECK( 24 ) = 1

  IF(ALLOCATED( FRANPREV ) .AND. RSLEVEL >= 1) REALCHECK( 25 ) = 1

  IF(ALLOCATED( FSCOUL ) .AND. RSLEVEL >= 1) REALCHECK( 26 ) = 1

  IF(ALLOCATED( FSSPIN ) .AND. RSLEVEL >= 1) REALCHECK( 27 ) = 1

  IF(ALLOCATED( FTOT ) .AND. RSLEVEL >= 1) REALCHECK( 28 ) = 1

  IF(ALLOCATED( H ) .AND. RSLEVEL >= 1) REALCHECK( 29 ) = 1

  IF(ALLOCATED( H0 ) .AND. RSLEVEL >= 1) REALCHECK( 30 ) = 1

  IF(ALLOCATED( H2VECT ) .AND. RSLEVEL >= 0) REALCHECK( 31 ) = 1

  IF(ALLOCATED( HDIAG ) .AND. RSLEVEL >= 0) REALCHECK( 32 ) = 1

  IF(ALLOCATED( HDOWN ) .AND. RSLEVEL >= 1) REALCHECK( 33 ) = 1

  IF(ALLOCATED( HED ) .AND. RSLEVEL >= 0) REALCHECK( 34 ) = 1

  IF(ALLOCATED( HEF ) .AND. RSLEVEL >= 0) REALCHECK( 35 ) = 1

  IF(ALLOCATED( HEP ) .AND. RSLEVEL >= 0) REALCHECK( 36 ) = 1

  IF(ALLOCATED( HES ) .AND. RSLEVEL >= 0) REALCHECK( 37 ) = 1

  IF(ALLOCATED( HJJ ) .AND. RSLEVEL >= 0) REALCHECK( 38 ) = 1

  IF(ALLOCATED( HR0 ) .AND. RSLEVEL >= 0) REALCHECK( 39 ) = 1

  IF(ALLOCATED( HUBBARDU ) .AND. RSLEVEL >= 0) REALCHECK( 40 ) = 1

  IF(ALLOCATED( HUP ) .AND. RSLEVEL >= 1) REALCHECK( 41 ) = 1

  IF(ALLOCATED( KEVALS ) .AND. RSLEVEL >= 1) REALCHECK( 42 ) = 1

  IF(ALLOCATED( LCNSHIFT ) .AND. RSLEVEL >= 0) REALCHECK( 43 ) = 1

  IF(ALLOCATED( MASS ) .AND. RSLEVEL >= 0) REALCHECK( 44 ) = 1

  IF(ALLOCATED( MYCHARGE ) .AND. RSLEVEL >= 0) REALCHECK( 45 ) = 1

  IF(ALLOCATED( NONOTMP ) .AND. RSLEVEL >= 1) REALCHECK( 46 ) = 1

  IF(ALLOCATED( NONO_EVALS ) .AND. RSLEVEL >= 0) REALCHECK( 47 ) = 1

  IF(ALLOCATED( NONO_WORK ) .AND. RSLEVEL >= 0) REALCHECK( 48 ) = 1

  IF(ALLOCATED( OLDDELTAQS ) .AND. RSLEVEL >= 0) REALCHECK( 49 ) = 1

  IF(ALLOCATED( OLDDELTASPIN ) .AND. RSLEVEL >= 0) REALCHECK( 50 ) = 1

  IF(ALLOCATED( ORTHOH ) .AND. RSLEVEL >= 1) REALCHECK( 51 ) = 1

  IF(ALLOCATED( ORTHOHDOWN ) .AND. RSLEVEL >= 1) REALCHECK( 52 ) = 1

  IF(ALLOCATED( ORTHOHUP ) .AND. RSLEVEL >= 1) REALCHECK( 53 ) = 1

  IF(ALLOCATED( ORTHORHO ) .AND. RSLEVEL >= 1) REALCHECK( 54 ) = 1

  IF(ALLOCATED( OVERL ) .AND. RSLEVEL >= 1) REALCHECK( 55 ) = 1

  IF(ALLOCATED( PAIR ) .AND. RSLEVEL >= 1) REALCHECK( 56 ) = 1

  IF(ALLOCATED( PHIST ) .AND. RSLEVEL >= 0) REALCHECK( 57 ) = 1

  IF(ALLOCATED( PHISTX ) .AND. RSLEVEL >= 0) REALCHECK( 58 ) = 1

  IF(ALLOCATED( PHISTY ) .AND. RSLEVEL >= 0) REALCHECK( 59 ) = 1

  IF(ALLOCATED( PHISTZ ) .AND. RSLEVEL >= 0) REALCHECK( 60 ) = 1

  IF(ALLOCATED( PNK ) .AND. RSLEVEL >= 1) REALCHECK( 61 ) = 1

  IF(ALLOCATED( POTCOEF ) .AND. RSLEVEL >= 1) REALCHECK( 62 ) = 1

  IF(ALLOCATED( PPR ) .AND. RSLEVEL >= 1) REALCHECK( 63 ) = 1

  IF(ALLOCATED( PPSPL ) .AND. RSLEVEL >= 1) REALCHECK( 64 ) = 1

  IF(ALLOCATED( PPVAL ) .AND. RSLEVEL >= 1) REALCHECK( 65 ) = 1

  IF(ALLOCATED( QLIST ) .AND. RSLEVEL >= 0) REALCHECK( 66 ) = 1

  IF(ALLOCATED( RESPCHI ) .AND. RSLEVEL >= 0) REALCHECK( 67 ) = 1

  IF(ALLOCATED( RHODOWN ) .AND. RSLEVEL >= 1) REALCHECK( 68 ) = 1

  IF(ALLOCATED( RHODOWNZERO ) .AND. RSLEVEL >= 0) REALCHECK( 69 ) = 1

  IF(ALLOCATED( RHOUP ) .AND. RSLEVEL >= 1) REALCHECK( 70 ) = 1

  IF(ALLOCATED( RHOUPZERO ) .AND. RSLEVEL >= 0) REALCHECK( 71 ) = 1

  IF(ALLOCATED( SH2 ) .AND. RSLEVEL >= 1) REALCHECK( 72 ) = 1

  IF(ALLOCATED( SINLIST ) .AND. RSLEVEL >= 0) REALCHECK( 73 ) = 1

  IF(ALLOCATED( SMAT ) .AND. RSLEVEL >= 1) REALCHECK( 74 ) = 1

  IF(ALLOCATED( SPINLIST ) .AND. RSLEVEL >= 0) REALCHECK( 75 ) = 1

  IF(ALLOCATED( SPINTMP ) .AND. RSLEVEL >= 1) REALCHECK( 76 ) = 1

  IF(ALLOCATED( SPIN_PNK ) .AND. RSLEVEL >= 1) REALCHECK( 77 ) = 1

  IF(ALLOCATED( THIST ) .AND. RSLEVEL >= 0) REALCHECK( 78 ) = 1

  IF(ALLOCATED( TMPBODIAG ) .AND. RSLEVEL >= 0) REALCHECK( 79 ) = 1

  IF(ALLOCATED( TMPRHODOWN ) .AND. RSLEVEL >= 0) REALCHECK( 80 ) = 1

  IF(ALLOCATED( TMPRHOUP ) .AND. RSLEVEL >= 0) REALCHECK( 81 ) = 1

  IF(ALLOCATED( TWOXX2 ) .AND. RSLEVEL >= 1) REALCHECK( 82 ) = 1

  IF(ALLOCATED( UMAT ) .AND. RSLEVEL >= 1) REALCHECK( 83 ) = 1

  IF(ALLOCATED( UPEVALS ) .AND. RSLEVEL >= 0) REALCHECK( 84 ) = 1

  IF(ALLOCATED( UPEVECS ) .AND. RSLEVEL >= 1) REALCHECK( 85 ) = 1

  IF(ALLOCATED( V ) .AND. RSLEVEL >= 1) REALCHECK( 86 ) = 1

  IF(ALLOCATED( VHIST ) .AND. RSLEVEL >= 0) REALCHECK( 87 ) = 1

  IF(ALLOCATED( WDD ) .AND. RSLEVEL >= 0) REALCHECK( 88 ) = 1

  IF(ALLOCATED( WFF ) .AND. RSLEVEL >= 0) REALCHECK( 89 ) = 1

  IF(ALLOCATED( WORK ) .AND. RSLEVEL >= 0) REALCHECK( 90 ) = 1

  IF(ALLOCATED( WPP ) .AND. RSLEVEL >= 0) REALCHECK( 91 ) = 1

  IF(ALLOCATED( WSS ) .AND. RSLEVEL >= 0) REALCHECK( 92 ) = 1

  IF(ALLOCATED( X2 ) .AND. RSLEVEL >= 1) REALCHECK( 93 ) = 1

  IF(ALLOCATED( X2DOWN ) .AND. RSLEVEL >= 1) REALCHECK( 94 ) = 1

  IF(ALLOCATED( X2HRHO ) .AND. RSLEVEL >= 1) REALCHECK( 95 ) = 1

  IF(ALLOCATED( X2UP ) .AND. RSLEVEL >= 1) REALCHECK( 96 ) = 1

  IF(ALLOCATED( XMAT ) .AND. RSLEVEL >= 1) REALCHECK( 97 ) = 1

  IF(ALLOCATED( ZHEEVD_RWORK ) .AND. RSLEVEL >= 0) REALCHECK( 98 ) = 1

  IF(ALLOCATED( FSLCN ) .AND. RSLEVEL >= 0) REALCHECK( 99 ) = 1

  WRITE(14)CHARACTERCHECK
  WRITE(14)COMPLEXCHECK
  WRITE(14)INTEGERCHECK
  WRITE(14)REALCHECK

  WRITE(14)ITER

  IF(CHARACTERCHECK( 2 ) == 1)THEN
     WRITE(14)SIZE( ATELE ,DIM=1)
     WRITE(14) ATELE
  ENDIF

  IF(CHARACTERCHECK( 3 ) == 1)THEN
     WRITE(14)SIZE( BASIS ,DIM=1)
     WRITE(14) BASIS
  ENDIF

  IF(CHARACTERCHECK( 4 ) == 1)THEN
     WRITE(14)SIZE( BTYPE ,DIM=1)
     WRITE(14) BTYPE
  ENDIF

  IF(CHARACTERCHECK( 5 ) == 1)THEN
     WRITE(14)SIZE( ELE ,DIM=1)
     WRITE(14) ELE
  ENDIF

  IF(CHARACTERCHECK( 6 ) == 1)THEN
     WRITE(14)SIZE( ELE1 ,DIM=1)
     WRITE(14) ELE1
  ENDIF

  IF(CHARACTERCHECK( 7 ) == 1)THEN
     WRITE(14)SIZE( ELE2 ,DIM=1)
     WRITE(14) ELE2
  ENDIF

  IF(CHARACTERCHECK( 8 ) == 1)THEN
     WRITE(14)SIZE( PPELE1 ,DIM=1)
     WRITE(14) PPELE1
  ENDIF

  IF(CHARACTERCHECK( 9 ) == 1)THEN
     WRITE(14)SIZE( PPELE2 ,DIM=1)
     WRITE(14) PPELE2
  ENDIF

  IF(COMPLEXCHECK( 2 ) == 1)THEN
     WRITE(14)SIZE( DIAG_ZWORK ,DIM=1)
     WRITE(14) DIAG_ZWORK
  ENDIF

  IF(COMPLEXCHECK( 3 ) == 1)THEN
     WRITE(14)SIZE( HK ,DIM=1) , SIZE( HK ,DIM=2) , SIZE( HK ,DIM=3)
     WRITE(14) HK
  ENDIF

  IF(COMPLEXCHECK( 4 ) == 1)THEN
     WRITE(14)SIZE( HK0 ,DIM=1) , SIZE( HK0 ,DIM=2) , SIZE( HK0 ,DIM=3)
     WRITE(14) HK0
  ENDIF

  IF(COMPLEXCHECK( 5 ) == 1)THEN
     WRITE(14)SIZE( HKDIAG ,DIM=1) , SIZE( HKDIAG ,DIM=2)
     WRITE(14) HKDIAG
  ENDIF

  IF(COMPLEXCHECK( 6 ) == 1)THEN
     WRITE(14)SIZE( KBO ,DIM=1) , SIZE( KBO ,DIM=2) , SIZE( KBO ,DIM=3)
     WRITE(14) KBO
  ENDIF

  IF(COMPLEXCHECK( 7 ) == 1)THEN
     WRITE(14)SIZE( KEVECS ,DIM=1) , SIZE( KEVECS ,DIM=2) , SIZE( KEVECS ,DIM=3)
     WRITE(14) KEVECS
  ENDIF

  IF(COMPLEXCHECK( 8 ) == 1)THEN
     WRITE(14)SIZE( KF ,DIM=1) , SIZE( KF ,DIM=2)
     WRITE(14) KF
  ENDIF

  IF(COMPLEXCHECK( 9 ) == 1)THEN
     WRITE(14)SIZE( KHTMP ,DIM=1) , SIZE( KHTMP ,DIM=2)
     WRITE(14) KHTMP
  ENDIF

  IF(COMPLEXCHECK( 10 ) == 1)THEN
     WRITE(14)SIZE( KORTHOH ,DIM=1) , SIZE( KORTHOH ,DIM=2) , SIZE( KORTHOH ,DIM=3)
     WRITE(14) KORTHOH
  ENDIF

  IF(COMPLEXCHECK( 11 ) == 1)THEN
     WRITE(14)SIZE( KXMAT ,DIM=1) , SIZE( KXMAT ,DIM=2) , SIZE( KXMAT ,DIM=3)
     WRITE(14) KXMAT
  ENDIF

  IF(COMPLEXCHECK( 12 ) == 1)THEN
     WRITE(14)SIZE( SK ,DIM=1) , SIZE( SK ,DIM=2) , SIZE( SK ,DIM=3)
     WRITE(14) SK
  ENDIF

  IF(COMPLEXCHECK( 13 ) == 1)THEN
     WRITE(14)SIZE( ZBO ,DIM=1) , SIZE( ZBO ,DIM=2)
     WRITE(14) ZBO
  ENDIF

  IF(COMPLEXCHECK( 14 ) == 1)THEN
     WRITE(14)SIZE( ZHEEVD_WORK ,DIM=1)
     WRITE(14) ZHEEVD_WORK
  ENDIF

  IF(COMPLEXCHECK( 15 ) == 1)THEN
     WRITE(14)SIZE( ZHJJ ,DIM=1)
     WRITE(14) ZHJJ
  ENDIF

  IF(INTEGERCHECK( 2 ) == 1)THEN
     WRITE(14)SIZE( DIAG_IWORK ,DIM=1)
     WRITE(14) DIAG_IWORK
  ENDIF

  IF(INTEGERCHECK( 3 ) == 1)THEN
     WRITE(14)SIZE( ELEMPOINTER ,DIM=1)
     WRITE(14) ELEMPOINTER
  ENDIF

  IF(INTEGERCHECK( 4 ) == 1)THEN
     WRITE(14)SIZE( IFAIL ,DIM=1)
     WRITE(14) IFAIL
  ENDIF

  IF(INTEGERCHECK( 5 ) == 1)THEN
     WRITE(14)SIZE( MATINDLIST ,DIM=1)
     WRITE(14) MATINDLIST
  ENDIF

  IF(INTEGERCHECK( 6 ) == 1)THEN
     WRITE(14)SIZE( MOLID ,DIM=1)
     WRITE(14) MOLID
  ENDIF

  IF(INTEGERCHECK( 7 ) == 1)THEN
     WRITE(14)SIZE( NEBCOUL ,DIM=1) , SIZE( NEBCOUL ,DIM=2) , SIZE( NEBCOUL ,DIM=3)
     WRITE(14) NEBCOUL
  ENDIF

  IF(INTEGERCHECK( 8 ) == 1)THEN
     WRITE(14)SIZE( NEBPP ,DIM=1) , SIZE( NEBPP ,DIM=2) , SIZE( NEBPP ,DIM=3)
     WRITE(14) NEBPP
  ENDIF

  IF(INTEGERCHECK( 9 ) == 1)THEN
     WRITE(14)SIZE( NEBTB ,DIM=1) , SIZE( NEBTB ,DIM=2) , SIZE( NEBTB ,DIM=3)
     WRITE(14) NEBTB
  ENDIF

  IF(INTEGERCHECK( 10 ) == 1)THEN
     WRITE(14)SIZE( NONO_IWORK ,DIM=1)
     WRITE(14) NONO_IWORK
  ENDIF

  IF(INTEGERCHECK( 11 ) == 1)THEN
     WRITE(14)SIZE( PPTABLENGTH ,DIM=1)
     WRITE(14) PPTABLENGTH
  ENDIF

  IF(INTEGERCHECK( 12 ) == 1)THEN
     WRITE(14)SIZE( RX ,DIM=1)
     WRITE(14) RX
  ENDIF

  IF(INTEGERCHECK( 13 ) == 1)THEN
     WRITE(14)SIZE( RXTMP ,DIM=1)
     WRITE(14) RXTMP
  ENDIF

  IF(INTEGERCHECK( 14 ) == 1)THEN
     WRITE(14)SIZE( SIGNLIST ,DIM=1)
     WRITE(14) SIGNLIST
  ENDIF

  IF(INTEGERCHECK( 15 ) == 1)THEN
     WRITE(14)SIZE( SPININDLIST ,DIM=1)
     WRITE(14) SPININDLIST
  ENDIF

  IF(INTEGERCHECK( 16 ) == 1)THEN
     WRITE(14)SIZE( TOTNEBCOUL ,DIM=1)
     WRITE(14) TOTNEBCOUL
  ENDIF

  IF(INTEGERCHECK( 17 ) == 1)THEN
     WRITE(14)SIZE( TOTNEBPP ,DIM=1)
     WRITE(14) TOTNEBPP
  ENDIF

  IF(INTEGERCHECK( 18 ) == 1)THEN
     WRITE(14)SIZE( TOTNEBTB ,DIM=1)
     WRITE(14) TOTNEBTB
  ENDIF

  IF(INTEGERCHECK( 19 ) == 1)THEN
     WRITE(14)SIZE( XB ,DIM=1)
     WRITE(14) XB
  ENDIF

  IF(INTEGERCHECK( 20 ) == 1)THEN
     WRITE(14)SIZE( ZHEEVD_IWORK ,DIM=1)
     WRITE(14) ZHEEVD_IWORK
  ENDIF

  IF(REALCHECK( 2 ) == 1)THEN
     WRITE(14)SIZE( ATOCC ,DIM=1)
     WRITE(14) ATOCC
  ENDIF

  IF(REALCHECK( 3 ) == 1)THEN
     WRITE(14)SIZE( BO ,DIM=1) , SIZE( BO ,DIM=2)
     WRITE(14) BO
  ENDIF

  IF(REALCHECK( 4 ) == 1)THEN
     WRITE(14)SIZE( BOND ,DIM=1) , SIZE( BOND ,DIM=2)
     WRITE(14) BOND
  ENDIF

  IF(REALCHECK( 5 ) == 1)THEN
     WRITE(14)SIZE( BOZERO ,DIM=1)
     WRITE(14) BOZERO
  ENDIF

  IF(REALCHECK( 6 ) == 1)THEN
     WRITE(14)SIZE( BO_PADDED ,DIM=1) , SIZE( BO_PADDED ,DIM=2)
     WRITE(14) BO_PADDED
  ENDIF

  IF(REALCHECK( 7 ) == 1)THEN
     WRITE(14)SIZE( CHEMPOT_PNK ,DIM=1)
     WRITE(14) CHEMPOT_PNK
  ENDIF

  IF(REALCHECK( 8 ) == 1)THEN
     WRITE(14)SIZE( COSLIST ,DIM=1)
     WRITE(14) COSLIST
  ENDIF

  IF(REALCHECK( 9 ) == 1)THEN
     WRITE(14)SIZE( COULOMBV ,DIM=1)
     WRITE(14) COULOMBV
  ENDIF

  IF(REALCHECK( 10 ) == 1)THEN
     WRITE(14)SIZE( CPLIST ,DIM=1)
     WRITE(14) CPLIST
  ENDIF

  IF(REALCHECK( 11 ) == 1)THEN
     WRITE(14)SIZE( CR ,DIM=1) , SIZE( CR ,DIM=2)
     WRITE(14) CR
  ENDIF

  IF(REALCHECK( 12 ) == 1)THEN
     WRITE(14)SIZE( DELTAQ ,DIM=1)
     WRITE(14) DELTAQ
  ENDIF

  IF(REALCHECK( 13 ) == 1)THEN
     WRITE(14)SIZE( DELTASPIN ,DIM=1)
     WRITE(14) DELTASPIN
  ENDIF

  IF(REALCHECK( 14 ) == 1)THEN
     WRITE(14)SIZE( DIAG_RWORK ,DIM=1)
     WRITE(14) DIAG_RWORK
  ENDIF

  IF(REALCHECK( 15 ) == 1)THEN
     WRITE(14)SIZE( DIAG_WORK ,DIM=1)
     WRITE(14) DIAG_WORK
  ENDIF

  IF(REALCHECK( 16 ) == 1)THEN
     WRITE(14)SIZE( DOWNEVALS ,DIM=1)
     WRITE(14) DOWNEVALS
  ENDIF

  IF(REALCHECK( 17 ) == 1)THEN
     WRITE(14)SIZE( DOWNEVECS ,DIM=1) , SIZE( DOWNEVECS ,DIM=2)
     WRITE(14) DOWNEVECS
  ENDIF

  IF(REALCHECK( 18 ) == 1)THEN
     WRITE(14)SIZE( EHIST ,DIM=1)
     WRITE(14) EHIST
  ENDIF

  IF(REALCHECK( 19 ) == 1)THEN
     WRITE(14)SIZE( EVALS ,DIM=1)
     WRITE(14) EVALS
  ENDIF

  IF(REALCHECK( 20 ) == 1)THEN
     WRITE(14)SIZE( EVECS ,DIM=1) , SIZE( EVECS ,DIM=2)
     WRITE(14) EVECS
  ENDIF

  IF(REALCHECK( 21 ) == 1)THEN
     WRITE(14)SIZE( F ,DIM=1) , SIZE( F ,DIM=2)
     WRITE(14) F
  ENDIF

  IF(REALCHECK( 22 ) == 1)THEN
     WRITE(14)SIZE( FCOUL ,DIM=1) , SIZE( FCOUL ,DIM=2)
     WRITE(14) FCOUL
  ENDIF

  IF(REALCHECK( 23 ) == 1)THEN
     WRITE(14)SIZE( FPP ,DIM=1) , SIZE( FPP ,DIM=2)
     WRITE(14) FPP
  ENDIF

  IF(REALCHECK( 24 ) == 1)THEN
     WRITE(14)SIZE( FPUL ,DIM=1) , SIZE( FPUL ,DIM=2)
     WRITE(14) FPUL
  ENDIF

  IF(REALCHECK( 25 ) == 1)THEN
     WRITE(14)SIZE( FRANPREV ,DIM=1) , SIZE( FRANPREV ,DIM=2)
     WRITE(14) FRANPREV
  ENDIF

  IF(REALCHECK( 26 ) == 1)THEN
     WRITE(14)SIZE( FSCOUL ,DIM=1) , SIZE( FSCOUL ,DIM=2)
     WRITE(14) FSCOUL
  ENDIF

  IF(REALCHECK( 27 ) == 1)THEN
     WRITE(14)SIZE( FSSPIN ,DIM=1) , SIZE( FSSPIN ,DIM=2)
     WRITE(14) FSSPIN
  ENDIF

  IF(REALCHECK( 28 ) == 1)THEN
     WRITE(14)SIZE( FTOT ,DIM=1) , SIZE( FTOT ,DIM=2)
     WRITE(14) FTOT
  ENDIF

  IF(REALCHECK( 29 ) == 1)THEN
     WRITE(14)SIZE( H ,DIM=1) , SIZE( H ,DIM=2)
     WRITE(14) H
  ENDIF

  IF(REALCHECK( 30 ) == 1)THEN
     WRITE(14)SIZE( H0 ,DIM=1) , SIZE( H0 ,DIM=2)
     WRITE(14) H0
  ENDIF

  IF(REALCHECK( 31 ) == 1)THEN
     WRITE(14)SIZE( H2VECT ,DIM=1)
     WRITE(14) H2VECT
  ENDIF

  IF(REALCHECK( 32 ) == 1)THEN
     WRITE(14)SIZE( HDIAG ,DIM=1)
     WRITE(14) HDIAG
  ENDIF

  IF(REALCHECK( 33 ) == 1)THEN
     WRITE(14)SIZE( HDOWN ,DIM=1) , SIZE( HDOWN ,DIM=2)
     WRITE(14) HDOWN
  ENDIF

  IF(REALCHECK( 34 ) == 1)THEN
     WRITE(14)SIZE( HED ,DIM=1)
     WRITE(14) HED
  ENDIF

  IF(REALCHECK( 35 ) == 1)THEN
     WRITE(14)SIZE( HEF ,DIM=1)
     WRITE(14) HEF
  ENDIF

  IF(REALCHECK( 36 ) == 1)THEN
     WRITE(14)SIZE( HEP ,DIM=1)
     WRITE(14) HEP
  ENDIF

  IF(REALCHECK( 37 ) == 1)THEN
     WRITE(14)SIZE( HES ,DIM=1)
     WRITE(14) HES
  ENDIF

  IF(REALCHECK( 38 ) == 1)THEN
     WRITE(14)SIZE( HJJ ,DIM=1)
     WRITE(14) HJJ
  ENDIF

  IF(REALCHECK( 39 ) == 1)THEN
     WRITE(14)SIZE( HR0 ,DIM=1)
     WRITE(14) HR0
  ENDIF

  IF(REALCHECK( 40 ) == 1)THEN
     WRITE(14)SIZE( HUBBARDU ,DIM=1)
     WRITE(14) HUBBARDU
  ENDIF

  IF(REALCHECK( 41 ) == 1)THEN
     WRITE(14)SIZE( HUP ,DIM=1) , SIZE( HUP ,DIM=2)
     WRITE(14) HUP
  ENDIF

  IF(REALCHECK( 42 ) == 1)THEN
     WRITE(14)SIZE( KEVALS ,DIM=1) , SIZE( KEVALS ,DIM=2)
     WRITE(14) KEVALS
  ENDIF

  IF(REALCHECK( 43 ) == 1)THEN
     WRITE(14)SIZE( LCNSHIFT ,DIM=1)
     WRITE(14) LCNSHIFT
  ENDIF

  IF(REALCHECK( 44 ) == 1)THEN
     WRITE(14)SIZE( MASS ,DIM=1)
     WRITE(14) MASS
  ENDIF

  IF(REALCHECK( 45 ) == 1)THEN
     WRITE(14)SIZE( MYCHARGE ,DIM=1)
     WRITE(14) MYCHARGE
  ENDIF

  IF(REALCHECK( 46 ) == 1)THEN
     WRITE(14)SIZE( NONOTMP ,DIM=1) , SIZE( NONOTMP ,DIM=2)
     WRITE(14) NONOTMP
  ENDIF

  IF(REALCHECK( 47 ) == 1)THEN
     WRITE(14)SIZE( NONO_EVALS ,DIM=1)
     WRITE(14) NONO_EVALS
  ENDIF

  IF(REALCHECK( 48 ) == 1)THEN
     WRITE(14)SIZE( NONO_WORK ,DIM=1)
     WRITE(14) NONO_WORK
  ENDIF

  IF(REALCHECK( 49 ) == 1)THEN
     WRITE(14)SIZE( OLDDELTAQS ,DIM=1)
     WRITE(14) OLDDELTAQS
  ENDIF

  IF(REALCHECK( 50 ) == 1)THEN
     WRITE(14)SIZE( OLDDELTASPIN ,DIM=1)
     WRITE(14) OLDDELTASPIN
  ENDIF

  IF(REALCHECK( 51 ) == 1)THEN
     WRITE(14)SIZE( ORTHOH ,DIM=1) , SIZE( ORTHOH ,DIM=2)
     WRITE(14) ORTHOH
  ENDIF

  IF(REALCHECK( 52 ) == 1)THEN
     WRITE(14)SIZE( ORTHOHDOWN ,DIM=1) , SIZE( ORTHOHDOWN ,DIM=2)
     WRITE(14) ORTHOHDOWN
  ENDIF

  IF(REALCHECK( 53 ) == 1)THEN
     WRITE(14)SIZE( ORTHOHUP ,DIM=1) , SIZE( ORTHOHUP ,DIM=2)
     WRITE(14) ORTHOHUP
  ENDIF

  IF(REALCHECK( 54 ) == 1)THEN
     WRITE(14)SIZE( ORTHORHO ,DIM=1) , SIZE( ORTHORHO ,DIM=2)
     WRITE(14) ORTHORHO
  ENDIF

  IF(REALCHECK( 55 ) == 1)THEN
     WRITE(14)SIZE( OVERL ,DIM=1) , SIZE( OVERL ,DIM=2)
     WRITE(14) OVERL
  ENDIF

  IF(REALCHECK( 56 ) == 1)THEN
     WRITE(14)SIZE( PAIR ,DIM=1) , SIZE( PAIR ,DIM=2)
     WRITE(14) PAIR
  ENDIF

  IF(REALCHECK( 57 ) == 1)THEN
     WRITE(14)SIZE( PHIST ,DIM=1)
     WRITE(14) PHIST
  ENDIF

  IF(REALCHECK( 58 ) == 1)THEN
     WRITE(14)SIZE( PHISTX ,DIM=1)
     WRITE(14) PHISTX
  ENDIF

  IF(REALCHECK( 59 ) == 1)THEN
     WRITE(14)SIZE( PHISTY ,DIM=1)
     WRITE(14) PHISTY
  ENDIF

  IF(REALCHECK( 60 ) == 1)THEN
     WRITE(14)SIZE( PHISTZ ,DIM=1)
     WRITE(14) PHISTZ
  ENDIF

  IF(REALCHECK( 61 ) == 1)THEN
     WRITE(14)SIZE( PNK ,DIM=1) , SIZE( PNK ,DIM=2)
     WRITE(14) PNK
  ENDIF

  IF(REALCHECK( 62 ) == 1)THEN
     WRITE(14)SIZE( POTCOEF ,DIM=1) , SIZE( POTCOEF ,DIM=2)
     WRITE(14) POTCOEF
  ENDIF

  IF(REALCHECK( 63 ) == 1)THEN
     WRITE(14)SIZE( PPR ,DIM=1) , SIZE( PPR ,DIM=2)
     WRITE(14) PPR
  ENDIF

  IF(REALCHECK( 64 ) == 1)THEN
     WRITE(14)SIZE( PPSPL ,DIM=1) , SIZE( PPSPL ,DIM=2)
     WRITE(14) PPSPL
  ENDIF

  IF(REALCHECK( 65 ) == 1)THEN
     WRITE(14)SIZE( PPVAL ,DIM=1) , SIZE( PPVAL ,DIM=2)
     WRITE(14) PPVAL
  ENDIF

  IF(REALCHECK( 66 ) == 1)THEN
     WRITE(14)SIZE( QLIST ,DIM=1)
     WRITE(14) QLIST
  ENDIF

  IF(REALCHECK( 67 ) == 1)THEN
     WRITE(14)SIZE( RESPCHI ,DIM=1)
     WRITE(14) RESPCHI
  ENDIF

  IF(REALCHECK( 68 ) == 1)THEN
     WRITE(14)SIZE( RHODOWN ,DIM=1) , SIZE( RHODOWN ,DIM=2)
     WRITE(14) RHODOWN
  ENDIF

  IF(REALCHECK( 69 ) == 1)THEN
     WRITE(14)SIZE( RHODOWNZERO ,DIM=1)
     WRITE(14) RHODOWNZERO
  ENDIF

  IF(REALCHECK( 70 ) == 1)THEN
     WRITE(14)SIZE( RHOUP ,DIM=1) , SIZE( RHOUP ,DIM=2)
     WRITE(14) RHOUP
  ENDIF

  IF(REALCHECK( 71 ) == 1)THEN
     WRITE(14)SIZE( RHOUPZERO ,DIM=1)
     WRITE(14) RHOUPZERO
  ENDIF

  IF(REALCHECK( 72 ) == 1)THEN
     WRITE(14)SIZE( SH2 ,DIM=1) , SIZE( SH2 ,DIM=2)
     WRITE(14) SH2
  ENDIF

  IF(REALCHECK( 73 ) == 1)THEN
     WRITE(14)SIZE( SINLIST ,DIM=1)
     WRITE(14) SINLIST
  ENDIF

  IF(REALCHECK( 74 ) == 1)THEN
     WRITE(14)SIZE( SMAT ,DIM=1) , SIZE( SMAT ,DIM=2)
     WRITE(14) SMAT
  ENDIF

  IF(REALCHECK( 75 ) == 1)THEN
     WRITE(14)SIZE( SPINLIST ,DIM=1)
     WRITE(14) SPINLIST
  ENDIF

  IF(REALCHECK( 76 ) == 1)THEN
     WRITE(14)SIZE( SPINTMP ,DIM=1) , SIZE( SPINTMP ,DIM=2)
     WRITE(14) SPINTMP
  ENDIF

  IF(REALCHECK( 77 ) == 1)THEN
     WRITE(14)SIZE( SPIN_PNK ,DIM=1) , SIZE( SPIN_PNK ,DIM=2)
     WRITE(14) SPIN_PNK
  ENDIF

  IF(REALCHECK( 78 ) == 1)THEN
     WRITE(14)SIZE( THIST ,DIM=1)
     WRITE(14) THIST
  ENDIF

  IF(REALCHECK( 79 ) == 1)THEN
     WRITE(14)SIZE( TMPBODIAG ,DIM=1)
     WRITE(14) TMPBODIAG
  ENDIF

  IF(REALCHECK( 80 ) == 1)THEN
     WRITE(14)SIZE( TMPRHODOWN ,DIM=1)
     WRITE(14) TMPRHODOWN
  ENDIF

  IF(REALCHECK( 81 ) == 1)THEN
     WRITE(14)SIZE( TMPRHOUP ,DIM=1)
     WRITE(14) TMPRHOUP
  ENDIF

  IF(REALCHECK( 82 ) == 1)THEN
     WRITE(14)SIZE( TWOXX2 ,DIM=1) , SIZE( TWOXX2 ,DIM=2)
     WRITE(14) TWOXX2
  ENDIF

  IF(REALCHECK( 83 ) == 1)THEN
     WRITE(14)SIZE( UMAT ,DIM=1) , SIZE( UMAT ,DIM=2)
     WRITE(14) UMAT
  ENDIF

  IF(REALCHECK( 84 ) == 1)THEN
     WRITE(14)SIZE( UPEVALS ,DIM=1)
     WRITE(14) UPEVALS
  ENDIF

  IF(REALCHECK( 85 ) == 1)THEN
     WRITE(14)SIZE( UPEVECS ,DIM=1) , SIZE( UPEVECS ,DIM=2)
     WRITE(14) UPEVECS
  ENDIF

  IF(REALCHECK( 86 ) == 1)THEN
     WRITE(14)SIZE( V ,DIM=1) , SIZE( V ,DIM=2)
     WRITE(14) V
  ENDIF

  IF(REALCHECK( 87 ) == 1)THEN
     WRITE(14)SIZE( VHIST ,DIM=1)
     WRITE(14) VHIST
  ENDIF

  IF(REALCHECK( 88 ) == 1)THEN
     WRITE(14)SIZE( WDD ,DIM=1)
     WRITE(14) WDD
  ENDIF

  IF(REALCHECK( 89 ) == 1)THEN
     WRITE(14)SIZE( WFF ,DIM=1)
     WRITE(14) WFF
  ENDIF

  IF(REALCHECK( 90 ) == 1)THEN
     WRITE(14)SIZE( WORK ,DIM=1)
     WRITE(14) WORK
  ENDIF

  IF(REALCHECK( 91 ) == 1)THEN
     WRITE(14)SIZE( WPP ,DIM=1)
     WRITE(14) WPP
  ENDIF

  IF(REALCHECK( 92 ) == 1)THEN
     WRITE(14)SIZE( WSS ,DIM=1)
     WRITE(14) WSS
  ENDIF

  IF(REALCHECK( 93 ) == 1)THEN
     WRITE(14)SIZE( X2 ,DIM=1) , SIZE( X2 ,DIM=2)
     WRITE(14) X2
  ENDIF

  IF(REALCHECK( 94 ) == 1)THEN
     WRITE(14)SIZE( X2DOWN ,DIM=1) , SIZE( X2DOWN ,DIM=2)
     WRITE(14) X2DOWN
  ENDIF

  IF(REALCHECK( 95 ) == 1)THEN
     WRITE(14)SIZE( X2HRHO ,DIM=1) , SIZE( X2HRHO ,DIM=2)
     WRITE(14) X2HRHO
  ENDIF

  IF(REALCHECK( 96 ) == 1)THEN
     WRITE(14)SIZE( X2UP ,DIM=1) , SIZE( X2UP ,DIM=2)
     WRITE(14) X2UP
  ENDIF

  IF(REALCHECK( 97 ) == 1)THEN
     WRITE(14)SIZE( XMAT ,DIM=1) , SIZE( XMAT ,DIM=2)
     WRITE(14) XMAT
  ENDIF

  IF(REALCHECK( 98 ) == 1)THEN
     WRITE(14)SIZE( ZHEEVD_RWORK ,DIM=1)
     WRITE(14) ZHEEVD_RWORK
  ENDIF

  IF(REALCHECK( 99 ) == 1)THEN
     WRITE(14)SIZE( FSLCN ,DIM=1), SIZE( FSLCN, DIM=2)
     WRITE(14) FSLCN
  ENDIF

  CLOSE(14)

END SUBROUTINE WRTRESTARTLIB
